#!/usr/bin/perl

#Usage: perl cvs2pl.pl -f HN_Tumor_and_Stage_Text.csv -o invasive_DCIS.yap [-b invasive_det.b]

#perl script similar to cvs2pl.py
#this script takes in a cvs file and converts it to prolog notation
#the first line of the cvs file is used as the predicate names
#all subsequent lines are taken as the actual data
#at the top of the output file determinations and modes will also be written.

#Dates are in US format: MM/DD/YY.
#It converts dates into %Y%m%d - 19961025 (%Q) [seconds from epoch (%s)]

#Separates TUMOR2 into a new record, and assigns a uniqueID to it.

use strict;
use warnings;
use Carp;
use Date::Manip;

#Below, each nb refers to the header index, starting from zero.
use constant ABNRMID => 2; #ABNRMID attribute nb in header
use constant DATE => '^(1|43)$'; #header attribute nbs that are dates
use constant SKIP => '^(2|3|10|29|30|31|32|33|35|36|37|38|39|40|41|42|44|45|46|47|48|49|50|51|52|53|56)$'; #header attribute nbs to skip
#uniqueID, Age, SvD, Radiologist,RINTERPRET, LINTERPRET, DISPOSITN, diffL, diffR, tumor1size, ..., biopsyLat, Biopsytype ... stage, report
use constant SKIPB => '^(0|1|5|6|7|12|13|34|35|36|37|38|39|40|41|43|54)$'; #additional attribute nbs to skip in .b file
#MNR, study date, prhx, prsr, palp, l, r birads, tumor1 info, tumor2 info, biopsy date, qualdx
use constant TEXT => 56; #text is last, cutoff. Makeshift solution for split /,/

#BIRADS side & location
use constant TUMOR1LOC => 34;
use constant TUMOR2LOC => 38;

#Tumor 2
#Highest sequential uniqueID: 148,144
use constant NEWID => 148145;
use constant TUMOR2TRIGGER => 380; #ATT disabeled
use constant TUMOR2 => '^(38|39|40|41)$'; #TUMOR2 ONLY attributes
use constant BOTHTUMORS => '^(0|1|12|13|14|43|55)$'; #attributes common to both tumors
#MNR, study date,BIRADS l,r,c, biopsy date, mssz

eval (&main());

sub main(){
    #process arguments
    my %args = @ARGV;
    croak "Usage: perl ".__FILE__." -f in_file.cvs -o out_file.yap [-b b_file.b],"
        unless $args{-f} && $args{-o};
    open(FILEIN, "$args{-f}") || 
	confess "couldn't open $args{-f}: $!. Stopped";
    open(FILEOUT,">$args{-o}") || 
	confess "couldn't open $args{-o}: $!. Stopped";
    my $b = 0; #print determination and mode .b file?
    if ($args{-b}){
	open(FILEB,">$args{-b}") || 
	    confess "couldn't open $args{-b}: $!. Stopped";
	$b = 1;
    }

    #process header to get determinations and modes
    my $line = <FILEIN>; #trick to store header
    my (@preds) = split /,/, $line; 

    print FILEB ":-modeh(1,invasive(+abnormalityID)).\n\n" if $b;
    foreach my $i (0..$#preds){
	$preds[$i] = &format($preds[$i], 1);
	if ($b && $i !~ SKIP && $i !~ SKIPB){
	    print FILEB ":-modeb(1,$preds[$i](+abnormalityID,#type_$preds[$i])).\n";
	    print FILEB ":-determination(invasive/1,$preds[$i]/2).\n";
	}
    }
    close (FILEB) if $b;

    #process each line
    my $newId = NEWID;
    while ($line = <FILEIN>){
	my (@elems) = split /,/, $line; 
	my $abnrmId = &format($elems[ABNRMID]);

	#process fields until text
	foreach my $i (0..TEXT-1){
	    next if $i =~ SKIP; #skip unwanted fields
	    next if $i =~ TUMOR2; #skip tumor2 fields
	    print FILEOUT "$preds[$i]($abnrmId,".
		&format(&processElements($i, \@elems, TUMOR1LOC)).").\n";
	    
	}

	#process second tumor
	if ($elems[TUMOR2TRIGGER]){
	    $newId++;
	    foreach my $i (0..TEXT-1){
		next unless $i =~ TUMOR2 || $i =~ BOTHTUMORS;
		print FILEOUT "$preds[$i]($newId,".
		    &format(&processElements($i, \@elems, TUMOR2LOC)).").\n";
	    }
	}
    }
    close (FILEIN);
    close (FILEOUT);
}


#process the values in each record. Check for dates, BIRADS sides
sub processElements(){
    my $i = shift;
    my $elems = shift;
    my $tumor = shift;
    my $value = $elems->[$i];
    
    #change date formats
    $value = UnixDate(ParseDate($elems->[$i]),"%Q") if ($i =~ DATE);

    return $value;
}



#removes \" and spaces. Remove '-' if between two characters
#Converts to lower case
#puts ' around a word if it starts with a digit followed by letters
#if header, remove also digits. So I unify tumor1 and tumor2 features
sub format(){
    my $word = shift;
    my $header = shift;
    $word = "noEnt" if $word =~ /^$/;
    $word =~ s/\d//g if $header;

    $word =~ s/[\"|\s]//g;
    $word =~ s/(\w)-(\w)/$1$2/g;
    $word = "'$word'" if $word =~ /^\d+\D/;
    return lc($word);
}



