#!/usr/bin/perl

# divides the dataset into folds, each fold containing all records 
# of a patient

# Usage: perl evenly_divide_by_age.pl in_file
# takes in HN_Tumor_and_Stage_Text.csv
# and outputs Aleph .f and .n example files, divided by age groups:
# strictly <50 and >=65.
# The output is divided by folds, .# where each # is the fold.
# Y: Younger, O: Older, S: inSitu, N: iNvasive

#ATT: assumes a particular order of attributes.

use strict;
no strict 'refs'; # so that symbolic references can be used
use warnings;
use Carp;

#Below, each nb refers to the header index, starting from zero.
use constant TARGET => 54; #the target, Invasive/InSitu
use constant AGE => 3;
use constant UNIQUEID => 2;
use constant MRN => 0; #to make sure all records of one patient in same fold

eval (&main());

sub main(){
    #process arguments
    my @args = @ARGV;
    croak "Usage: perl ", __FILE__," in_file"
	unless @args == 1;
    open(FILEIN, "$args[0]") || confess "couldn't open $args[0]: $!. Stopped";
    my %HoH = ("YN"=>{}, "YS"=>{}, "MN"=>{}, "MS"=>{}, "ON"=>{}, "OS"=>{}); 
    #Young Old Middle iNvasive inSitu

    #classify records
    <FILEIN>; #get rid of first line
    my ($fh, $age, $target, $uniqueID, $MRN);
    while (my $line = <FILEIN>){
	$fh = ''; #reset it
	my (@elems) = split /,/, $line;
	$target = $elems[TARGET];
	$uniqueID = $elems[UNIQUEID];
	$MRN = $elems[MRN];

	#Take last two digits of ageGroup (Age6569; AgeLT45...) 
	$elems[AGE] =~ /(\d\d)\"$/;
	$age = $1;

	if ($target =~ /Invasive/){
	    if ($age < 50) {$fh = "YN";}
	    if ($age >= 50 && $age < 65) {$fh = "MN";}
	    if ($age >= 65) {$fh = "ON";}
	}
	if ($target =~ /InSitu/){
	    if ($age < 50) {$fh = "YS";}
	    if ($age >= 50 && $age < 65) {$fh = "MS";}
	    if ($age >= 65) {$fh = "OS";}
	}	
	&addRecord($HoH{$fh}, $MRN, $uniqueID) if $fh;
    }

    #print files. 2 is because we want two folds
    &printFile($HoH{$_}, $_, 2)
	foreach ("YN","YS","MN","MS","ON","OS");
	
    close (FILEIN);
}

#given a hash and an element, add it and update count
sub addRecord{
    my $h = shift;
    my $MRN = shift;
    (defined $h->{$MRN}) ?
	($h->{$MRN}->addOccurrence(@_)) :
	($h->{$MRN} = Record->new(@_));
}

#divide into folds and print
sub printFile{
    my $h = shift;
    my $fh = shift;
    my $nb = shift;

    my @total = (0) x $nb; #total nb of abnormalities in each fold
    my @AoH; #AoH, each hash is a fold

    my $currentFold = 0;
    #sort main hash in decreasing order of occurrences
    foreach my $key (sort {$h->{$b}->occurrenceNb <=> $h->{$a}->occurrenceNb} 
		     (keys %$h)) {
	#search for correct fold
	$currentFold = ($currentFold+1) % $nb
	    if $total[$currentFold] >= $total[($currentFold-1) % $nb];
	#assign to fold
	$AoH[$currentFold]{$key} = $h->{$key}; 
	$total[$currentFold] += $h->{$key}->occurrenceNb;
	#print "$currentFold\t".$AoH[$currentFold]{$key}->occurrenceNb."\t$key\n";
    }

    #print to files
    foreach my $fold (0..$nb-1){
	open(FH, ">$fh.$fold") || confess "couldn't open a file: $!. Stopped";
	#print "Fold $fold\t $total[$fold]\n";
	#for each patient
	foreach my $MRN (keys %{$AoH[$fold]} ) {
	    #for each mammography
	    foreach my $mammoID ($h->{$MRN}->mammoID()){
		#foreach abnormality
		print FH "invasive($mammoID).\n";
	    }
	}
	close(FH);
    }
}


#class Record
{
    package Record;

    sub new{
	my $class = shift;
	my $self = {};
	$self->{NB} = 0; #cumulative nb of occurrences
	bless $self, $class;
	$self->addOccurrence(@_);
	return $self;
    }

    #given mammo ID, adds a new occurrence, 
    sub addOccurrence{
	my $self = shift;
	my $mammoID = shift;

	$self->{NB}++; #increment total nb
	#add abnormality, and mammoID if needed
	$self->{MRN}->{$mammoID} = $mammoID;
    }

    #returns the number of accurrences
    sub occurrenceNb{my $self = shift; return $self->{NB}; }

    #return array of mammoID
    sub mammoID{my $self = shift; return keys %{$self->{MRN}} };

1;    
}



